CREATE SEQUENCE PROC_DATA_SEQUENCE MINVALUE 1 NOMAXVALUE ORDER;

CREATE SEQUENCE EVENT_SEQUENCE MINVALUE 1 NOMAXVALUE ORDER;

ALTER TABLE "HS_AUDIT_TRAIL"
ADD("EVENT_SEQ" DECIMAL(16,0));

ALTER TABLE "HS_PROC_DATA"
ADD ("PROC_DATA_SEQ" DECIMAL(16,0));

ALTER TABLE "HS_CONT_DETAIL"
ADD("CONT_DETAIL_SEQ" DECIMAL(16,0));

CREATE OR REPLACE TRIGGER "AT_INST" AFTER INSERT ON "AUDIT_TRAIL" REFERENCING NEW AS NEW_REC FOR EACH ROW 
 BEGIN
 INSERT INTO HS_AUDIT_TRAIL
 (EVENT_SEQ, CREATED, EVENT, TEMPL_VALID_FROM, PROCESS_NAME, PROCESS_ID, TOP_LVL_PROC_NAME,
 TOP_LVL_PROC_ID, PARENT_PROC_NAME, PARENT_PROC_ID, PROC_TEMPL_NAME, BLOCK_NAMES,
 USER_NAME, SECOND_USER_NAME, ACTIVITY_NAME, ACTIVITY_TYPE, ACTIVITY_STATE,
 COMMAND_PARAMETERS, PROGRAM_NAME, ACTIVITY_RC, ASSOCIATED_OBJECT,
 OBJECT_DESCRIPTION, SECOND_ACT_NAME)
 VALUES
 (EVENT_SEQUENCE.NEXTVAL, :NEW_REC.CREATED, :NEW_REC.EVENT, :NEW_REC.TEMPL_VALID_FROM,
 :NEW_REC.PROCESS_NAME, :NEW_REC.PROCESS_ID, :NEW_REC.TOP_LVL_PROC_NAME,
 :NEW_REC.TOP_LVL_PROC_ID, :NEW_REC.PARENT_PROC_NAME, :NEW_REC.PARENT_PROC_ID,
 :NEW_REC.PROC_TEMPL_NAME, :NEW_REC.BLOCK_NAMES, :NEW_REC.USER_NAME,
 :NEW_REC.SECOND_USER_NAME, :NEW_REC.ACTIVITY_NAME, :NEW_REC.ACTIVITY_TYPE,
 :NEW_REC.ACTIVITY_STATE, :NEW_REC.COMMAND_PARAMETERS, :NEW_REC.PROGRAM_NAME,
 :NEW_REC.ACTIVITY_RC, :NEW_REC.ASSOCIATED_OBJECT, :NEW_REC.OBJECT_DESCRIPTION,
 :NEW_REC.SECOND_ACT_NAME);
 END;
/
CREATE OR REPLACE TRIGGER "PD_INST" AFTER INSERT ON "CONTAINER" REFERENCING NEW AS PD FOR EACH ROW 
 BEGIN
 INSERT INTO
 HS_PROC_DATA(PROC_DATA_SEQ, BIID, CIID, LAST_UPDATED, SHORT_DATA)
 VALUES
 (PROC_DATA_SEQUENCE.NEXTVAL, :PD.BIID, :PD.CIID, :PD.LAST_UPDATED, :PD.SHORT_DATA);
 END;
/
CREATE OR REPLACE TRIGGER "PDD_INST" AFTER INSERT ON "CONTAINER_DATA_SEG" REFERENCING NEW AS PD FOR EACH ROW 
 BEGIN
 INSERT INTO
 HS_CONT_DETAIL(CONT_DETAIL_SEQ, CD_BIID, CD_CIID, CD_LAST_UPDATED, SEQ_NUM, DATA)
 VALUES
 (PROC_DATA_SEQUENCE.NEXTVAL, :PD.BIID, :PD.CIID, TIMESTAMP_WF(SYSDATE, 0), :PD.SEQ_NUM, :PD.DATA);
 END;
/
CREATE OR REPLACE TRIGGER "PD_UPDT" AFTER UPDATE ON "CONTAINER" REFERENCING NEW AS PD FOR EACH ROW 
 BEGIN
 INSERT INTO HS_PROC_DATA(PROC_DATA_SEQ, BIID, CIID, LAST_UPDATED, SHORT_DATA)
 VALUES
 (PROC_DATA_SEQUENCE.NEXTVAL, :PD.BIID, :PD.CIID, :PD.LAST_UPDATED, :PD.SHORT_DATA);
 END;
/
CREATE OR REPLACE TRIGGER "PDD_UPDT" AFTER UPDATE ON "CONTAINER_DATA_SEG" REFERENCING NEW AS PD FOR EACH ROW 
 BEGIN
 INSERT INTO
 HS_CONT_DETAIL(CONT_DETAIL_SEQ, CD_BIID, CD_CIID, CD_LAST_UPDATED, SEQ_NUM, DATA)
 VALUES
 (PROC_DATA_SEQUENCE.NEXTVAL, :PD.BIID, :PD.CIID, TIMESTAMP_WF(SYSDATE, 0), :PD.SEQ_NUM, :PD.DATA);
 END;
/

DROP SEQUENCE TIMESTAMP_SEQ ;

CREATE OR REPLACE PROCEDURE UPDATE_HS_AUDIT_DATA
IS
v_counter number;
v_total number;
CURSOR C1 IS SELECT DISTINCT A.CREATED.D DT, A.CREATED.S FR, A.EVENT EV
		FROM HS_AUDIT_TRAIL A
		ORDER BY A.CREATED.D, A.CREATED.S, A.EVENT;
BEGIN
	SELECT COUNT(*) INTO v_total FROM HS_AUDIT_TRAIL;
	v_counter := 1;
	FOR REC1 IN C1 LOOP
		UPDATE HS_AUDIT_TRAIL A SET EVENT_SEQ = -1 * v_total
			WHERE A.CREATED.D = REC1.DT AND A.CREATED.S = REC1.FR AND A.EVENT = REC1.EV;
		v_total := v_total - 1;
		v_counter := v_counter + 1;
		IF (v_counter > 1000) THEN 
		COMMIT;
		v_counter := 1;
		END IF;
	END LOOP;
	COMMIT;
END;
/

CREATE OR REPLACE PROCEDURE UPDATE_HS_PROC_DATA
IS
v_counter number;
v_total number;
CURSOR C1 IS SELECT DISTINCT A.BIID,A.CIID,A.LAST_UPDATED.D DT, A.LAST_UPDATED.S FR
FROM HS_PROC_DATA A
ORDER BY A.BIID,A.CIID,A.LAST_UPDATED.D, A.LAST_UPDATED.S;
BEGIN
SELECT COUNT(*) INTO v_total FROM HS_PROC_DATA;
v_counter := 1;
FOR REC1 IN C1
LOOP
UPDATE HS_PROC_DATA A SET PROC_DATA_SEQ = -1 * v_total
WHERE A.BIID=REC1.BIID AND A.CIID = REC1.CIID AND A.LAST_UPDATED.D = REC1.DT AND A.LAST_UPDATED.S = REC1.FR;
v_total := v_total - 1;
v_counter := v_counter + 1;
IF (v_counter > 1000)
THEN
COMMIT;
v_counter := 1;
END IF;
END LOOP;
COMMIT;
END;
/

CREATE OR REPLACE PROCEDURE UPDATE_HS_CONT_DETAIL
IS
v_counter number;
v_total number;
CURSOR C1 IS SELECT DISTINCT A.CD_BIID,A.CD_CIID,A.CD_LAST_UPDATED.D DT, A.CD_LAST_UPDATED.S FR,A.SEQ_NUM
FROM HS_CONT_DETAIL A 
ORDER BY A.CD_BIID,A.CD_CIID,A.CD_LAST_UPDATED.D, A.CD_LAST_UPDATED.S,A.SEQ_NUM;
BEGIN
SELECT COUNT(*) INTO v_total FROM HS_CONT_DETAIL;
v_counter := 1;
FOR REC1 IN C1
LOOP
UPDATE HS_CONT_DETAIL A SET CONT_DETAIL_SEQ = -1 * v_total
WHERE A.CD_BIID=REC1.CD_BIID AND A.CD_CIID = REC1.CD_CIID AND A.CD_LAST_UPDATED.D = REC1.DT AND A.CD_LAST_UPDATED.S = REC1.FR AND A.SEQ_NUM = REC1.SEQ_NUM ;
v_total := v_total - 1;
v_counter := v_counter + 1;
IF (v_counter > 1000)
THEN
COMMIT;
v_counter := 1;
END IF;
END LOOP;
COMMIT;
END;
/

CREATE INDEX "HS_Created_Index" ON HS_AUDIT_TRAIL(CREATED.D,CREATED.S,EVENT);

ANALYZE TABLE HS_AUDIT_TRAIL COMPUTE STATISTICS;

EXECUTE UPDATE_HS_AUDIT_DATA()

DROP INDEX "HS_Created_Index";

CREATE INDEX "HS_AT_SEQ_INDEX" ON "HS_AUDIT_TRAIL"("EVENT_SEQ");

ANALYZE TABLE HS_AUDIT_TRAIL COMPUTE STATISTICS;


CREATE INDEX "HS_PD_TIME_INDEX" ON "HS_PROC_DATA"("LAST_UPDATED".D, "LAST_UPDATED".S);

ANALYZE TABLE "HS_PROC_DATA" COMPUTE STATISTICS;

EXECUTE UPDATE_HS_PROC_DATA()

CREATE INDEX "HS_PD_SEQ_INDEX" ON "HS_PROC_DATA"("PROC_DATA_SEQ");

ANALYZE TABLE "HS_PROC_DATA" COMPUTE STATISTICS;


CREATE INDEX "HS_PDD_TIME_INDEX" ON "HS_CONT_DETAIL"("CD_LAST_UPDATED".D, "CD_LAST_UPDATED".S);

ANALYZE TABLE "HS_CONT_DETAIL" COMPUTE STATISTICS;

EXECUTE UPDATE_HS_CONT_DETAIL()

CREATE INDEX "HS_PDD_SEQ_INDEX" ON "HS_CONT_DETAIL"("CONT_DETAIL_SEQ");

ANALYZE TABLE "HS_CONT_DETAIL" COMPUTE STATISTICS;

DROP PROCEDURE UPDATE_HS_AUDIT_DATA;

DROP PROCEDURE UPDATE_HS_PROC_DATA;

DROP PROCEDURE UPDATE_HS_CONT_DETAIL;

COMMIT;